require "set_up_development/version"
require "set_up_development/engine"

module SetUpDevelopment
  CHECK_MARK = "\u2713".freeze
  X_MARK = "\u2717".freeze

  class << self
    def list(config = nil)
      engine = Engine.new(config)
      summarize(engine)
    end

    def install!(config = nil)
      engine = Engine.new(config)

      engine.missing_dependencies.each do |dependency|
        if ask!("Install #{dependency.name}")
          dependency.install!
          puts "Installed"
        else
          puts "Deferred"
        end
      end

      summarize(engine)
    end

    def ask!(question)
      print "#{question} (y/n)? "
      yes_or_no = $stdin.gets
      yes_or_no.match(/^y/)
    end

    private

    def summarize(engine)
      engine.dependencies.each do |dependency|
        color, indicator = (dependency.installed? ? [:green, CHECK_MARK] : [:red, X_MARK])
        puts "#{indicator} #{dependency.name}".send(color)
      end
    end
  end
end
