require "yaml"
require "set_up_development/registry"
require "set_up_development/dependency"

module SetUpDevelopment
  class Engine
    DEFAULT_CONFIG = "set-up-development".freeze

    attr_accessor :registry,
      :config,
      :dependencies

    def initialize(config = nil)
      ENV.delete("RUBYOPT")
      self.config = YAML::load_file(config || DEFAULT_CONFIG) || {}
      define_dependencies!
    rescue Errno::ENOENT => load_error
      raise "No config file found. Try running `touch set-up-development`"
    end

    def missing_dependencies
      dependencies.select do |dependency|
        !dependency.installed?
      end
    end

    private

    def define_dependencies!
      self.dependencies = config.inject([]) do |dependencies, (installer_name, dependency_names)|
        installer = Registry.installers[installer_name]
        raise "No installer found for '#{installer_name}'" unless installer
        dependency_names.each do |dependency_name|
          dependencies << Dependency.new(installer, dependency_name)
        end
        dependencies
      end
    end
  end
end
